////////////////////////////////////////////
//
//      MIRRORS MODULE
//      For AGS 3.2.1+
//
//      Authors : The 'Indiana Jones and the Seven Cities of Gold' team
//                (contact: 'Monsieur Ouxx' on the AGS forums)
//
//      Dependencies : 'unpacked sprites' module
//      License : GPL. In a nutshell: Just credit us and you can do whatever you want.
//
//      HOW TO USE
//      - make sure the 'unpacked sprite' module is there, BEFORE 'mirrors'
//      - create the following elements in your room:
//        o Create a region (e.g. region #2). When the player walks on it, it will
//          turn on the mirror. When he walks off, it turns off the mirror.
//        o Create an object (with a sprite) for the mirror: 
//          Any non-transparent pixel in the object will reflect the character
//        o Create another object (with any sprite, or no sprite at all). Its CENTER
//          will be the "center" of the mirror. All symmetries are calculated from
//          that point.
//      - Choose a symmetry type from the enum below.
//      - Choose an opacity for the mirror. Note: Opacity 1.0 means the wall does not reflect anything!
//      - Choose arbitrary horizontal and vertical offsets (you don't have to, 
//        it's only if you don't want the reflection to be perfectly in line 
//        with the character.
//      - In "room_load" (that is: Event 'first time enters room'), declare the
//        mirror. It must be done ONLY ONCE per game.
//        Example :   Mirrors.NewMirror2(oMirror5,  oMirrorCenter5, region[1], MIRROR_VERTIC, 0.15,  10,  2);
//
//      ADDITIONAL NOTES:
//      - You will notice a very short freeze when the player enters the room.
//        That's because the module does a little bit of pre-processing to
//        enhance performance.
//      - The module cleans after itself when the player leaves the room.
//
//////////////////////////////////////////////////////


//#define MIRROR int
enum MirrorsSymetries{
  MIRROR_HORIZ, //the mirror is a plane facing the gamer
  MIRROR_VERTIC,  //the mirror is turned 90 from the gamer's viewpoint
  MIRROR_UPSIDEDOWN //the player will be reflected upside down, like in a lake
};

#define LOOP_DOWN  0
#define LOOP_LEFT  1
#define LOOP_RIGHT 2
#define LOOP_UP    3

struct Mirrors 
{
 
  //Declares a new mirror in the current room.
  //This must be done only once in the entire game (in "first time enters room")
  // It still has to be enabled afterwards to be visible.
  // 'mirror' is the object containing the sprite used as the mask
  // 'center' is a marker. The center of its sprite (width/2, height/2) should be where the player would stand (his feet coordinates) if he was glued to the mirror ;)
  // 'symetry' is one of the symetries in the enum above
  // 'x_offset' and 'y_offset' allow to shit the reflection by a constant number of pixels
  // 'opacity' must be in 0.0, 1.0. Note: if the mirror's opcaity is 1.0 then it's fully opaque, therefore there's NO reflection!
  import static void NewMirror2( Object* mirror, Object* center, Region* reg,  MirrorsSymetries symetry,  float opacity,  int x_offset=0,  int y_offset=0, int imperfections=0);

  //That doesn't delete the mirrors in memory (which is done automaticaly). It just disables them so that they're not rendered
  import static void DisableAllMirrors();
  
  //Enables all mirrors in THIS room if they've been previously declared with 'NewMirror'
  import static void EnableAllMirrorsInRoom();
  
  //Enables one mirror selectively
  import static void EnableMirror (Object* mirror);
};
